import math

class CollisionBase(object):

    def add_vectors(self, v1, v2):
        return [v1[0]+v2[0],v1[1]+v2[1],v1[2]+v2[2]]

    def sub_vectors(self, v1, v2):
        return [v1[0]-v2[0],v1[1]-v2[1],v1[2]-v2[2]]

    def average_vector(self, vec):
        return (vec[0]+vec[1]+vec[2])/3

    def get_distance(self, other):
        distance = ((self.pos[0]-other[0])**2)+\
                   ((self.pos[1]-other[1])**2)+\
                   ((self.pos[2]-other[2])**2)
        return math.sqrt(distance)

    def push(self, vel):
        self.velocity=self.add_vectors(self.velocity, vel)

    def pull(self, vel):
        if self.velocity[0] >= 0:
            self.velocity[0]-=vel[0]
            if self.velocity[0] < 0:
                self.velocity[0]=0
        else:
            self.velocity[0]+=vel[0]
            if self.velocity[0] > 0:
                self.velocity[0]=0

        if self.velocity[1] >= 0:
            self.velocity[1]-=vel[1]
            if self.velocity[1] < 0:
                self.velocity[1]=0
        else:
            self.velocity[1]+=vel[1]
            if self.velocity[1] > 0:
                self.velocity[1]=0

        if self.velocity[2] >= 0:
            self.velocity[2]-=vel[2]
            if self.velocity[2] < 0:
                self.velocity[2]=0
        else:
            self.velocity[2]+=vel[2]
            if self.velocity[2] > 0:
                self.velocity[2]=0

    def move(self, m):
        self.old_pos=list(self.pos)
        self.pos=self.add_vectors(self.pos, m)

    def apply_friction(self):
        f0=self.velocity[0]*self.friction
        f1=self.velocity[1]*self.friction
        f2=self.velocity[2]*self.friction
        self.pull([f0,f1,f2])

        if abs(self.velocity[0])<0.0001:
            self.velocity[0]=0
        if abs(self.velocity[1])<0.0001:
            self.velocity[1]=0
        if abs(self.velocity[2])<0.0001:
            self.velocity[2]=0

    def update(self):
        self.old_pos=list(self.pos)
        self.pos=self.add_vectors(self.pos, self.velocity)

        self.apply_friction()

    def get_abs_velocity(self):
        return [abs(self.velocity[0]),
                abs(self.velocity[1]),
                abs(self.velocity[2])]

    def back_track(self):
        self.pos=self.old_pos

class Sphere(CollisionBase):
    def __init__(self, pos, radius, friction, bouncy=False):
        self.radius=radius
        self.pos=pos
        self.old_pos=list(self.pos)
        self.friction=friction
        self.bouncy=bouncy

        self.type="circle"

        self.velocity=[0,0,0]

    def get_height(self):
        return self.radius

    def check_collision(self, other):
        if other.type=="circle":
            if self.get_distance(other.pos) <= self.radius+other.radius:
                return True
            return False
        else:
            return other.check_collision(self)

class BBox(CollisionBase):
    def __init__(self, pos, size, friction, bouncy=False):
        self.pos = pos
        self.old_pos=list(self.pos)

        self.sides=size
        self.friction=friction

        self.bouncy=bouncy

        self.type="bbox"

        self.velocity=[0,0,0]
        self.get_points()

    def get_height(self):
        return self.sides[1]

    def get_points(self):
        self.points=[[self.pos[0]-self.sides[0], self.pos[1]-self.sides[1], self.pos[2]-self.sides[2]],
                     [self.pos[0]+self.sides[0], self.pos[1]-self.sides[1], self.pos[2]-self.sides[2]],
                     [self.pos[0]-self.sides[0], self.pos[1]+self.sides[1], self.pos[2]-self.sides[2]],
                     [self.pos[0]+self.sides[0], self.pos[1]+self.sides[1], self.pos[2]-self.sides[2]],

                     [self.pos[0]-self.sides[0], self.pos[1]-self.sides[1], self.pos[2]+self.sides[2]],
                     [self.pos[0]+self.sides[0], self.pos[1]-self.sides[1], self.pos[2]+self.sides[2]],
                     [self.pos[0]-self.sides[0], self.pos[1]+self.sides[1], self.pos[2]+self.sides[2]],
                     [self.pos[0]+self.sides[0], self.pos[1]+self.sides[1], self.pos[2]+self.sides[2]]]

    def check_collision(self, other):
        if other.type=="circle":
            return (self.pos[0]-(self.sides[0]+other.radius)<=other.pos[0] and\
                    self.pos[0]+(self.sides[0]+other.radius)>=other.pos[0] and\

                    self.pos[1]-(self.sides[1]+other.radius)<=other.pos[1] and\
                    self.pos[1]+(self.sides[1]+other.radius)>=other.pos[1] and\

                    self.pos[2]-(self.sides[2]+other.radius)<=other.pos[2] and\
                    self.pos[2]+(self.sides[2]+other.radius)>=other.pos[2])
        if other.type=="bbox":
            for i in other.points:
                if (self.pos[0]-self.sides[0]<=i[0] and\
                    self.pos[0]+self.sides[0]>=i[0] and\

                    self.pos[1]-self.sides[1]<=i[1] and\
                    self.pos[1]+self.sides[1]>=i[1] and\

                    self.pos[2]-self.sides[2]<=i[2] and\
                    self.pos[2]+self.sides[2]>=i[2]):
                    return True

            for i in self.points:
                if (other.pos[0]-other.sides[0]<=i[0] and\
                    other.pos[0]+other.sides[0]>=i[0] and\

                    other.pos[1]-other.sides[1]<=i[1] and\
                    other.pos[1]+other.sides[1]>=i[1] and\

                    other.pos[2]-other.sides[2]<=i[2] and\
                    other.pos[2]+other.sides[2]>=i[2]):
                    return True
            return False
        else:
            return other.check_collision(self)

class newWorld(object):
    def __init__(self, gravity, friction, floor=0):
        self.gravity=gravity
        self.friction=friction
        self.floor=floor
